﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace APIMethodsCall
{
    public partial class frmAPIMethodsCall : Form
    {
        public frmAPIMethodsCall()
        {
            InitializeComponent();
        }

        #region Variables for API
        public enum enmAPIList
        {
            GET_SITE_LIST = 1,
            GET_FILE_INFO = 2,
            GET_FOLDER_INFO = 3,
            GET_FOLDERS_LIST = 4,
            GET_FILES_LIST = 5,
            ADD_FOLDER = 6,
            UPDATE_FOLDER = 7,
            DELETE_FOLDER = 8,
            ADD_FILE = 9,
            ADD_FILE_VERSION = 10,
            UPDATE_FILE = 11,
            DELETE_FILE = 12,
            DOWNLOAD_FILE = 13,
            RESTORE_FILE = 14,
            RESTORE_FOLDER = 15,
            MOVE_FILE = 16,
            MOVE_FOLDER = 17,
            GET_PROGRESS_KEY = 18,
            GET_PROGRESS_KEY_STATUS = 19,
            GET_CHANGES_LIST = 20,
            OAUTH = 21 //,
            //QUICK_SHARE = 22,
            //SHARE_VIA_LINK = 23
        }

        public static class RequestType
        {
            public static string Get = "GET";
            public static string Post = "POST";
            public static string Put = "PUT";
            public static string Delete = "DELETE";
        }

        public const string strAuthorizationKeyName = "Authorization";
        public const string strXMLContentType = "application/xml";
        public const string strUrlEncoded = "application/x-www-form-urlencoded";
        public const string strBearer = "Bearer ";
        public const string SYNC_STATUS_DONE = "DONE";
        public const string SYNC_STATUS_FAIL = "FAIL";

        public const string GET_SITE_LIST = "sites";
        public const string GET_FILE_INFO = "files/%%%fileid%%%";
        public const string GET_FOLDER_INFO = "folders/%%%folderid%%%";
        public const string GET_FOLDERS_LIST = "folders/%%%folderid%%%/items";
        public const string GET_FILES_LIST = "files?q=\"folderID=%%%folderid%%%\"";
        public const string ADD_FOLDER = "folders?rootfolderid=%%%rootfolderid%%%&parentfolderid=%%%parentfolderid%%%&sourceclient=%%%sourceclient%%%";
        public const string UPDATE_FOLDER = "folders/%%%folderid%%%?sourceclient=%%%sourceclient%%%";
        public const string DELETE_FOLDER = "folders/%%%folderid%%%?sourceclient=%%%sourceclient%%%";
        public const string ADD_FILE = "files\\content?rootfolderid=%%%rootfolderid%%%&parentfolderid=%%%parentfolderid%%%&sourceclient=%%%sourceclient%%%";
        public const string ADD_FILE_VERSION = "files\\%%%fileid%%%\\content?rootfolderid=%%%rootfolderid%%%&sourceclient=%%%sourceclient%%%";
        public const string UPDATE_FILE = "files/%%%fileid%%%?sourceclient=%%%sourceclient%%%";
        public const string DELETE_FILE = "files/%%%fileid%%%?sourceclient=%%%sourceclient%%%";
        public const string DOWNLOAD_FILE = "files/%%%fileid%%%/content";
        public const string RESTORE_FILE = "files/%%%fileid%%%/recycle?targetfolder=%%%targetfolderid%%%&sourceclient=%%%sourceclient%%%";
        public const string RESTORE_FOLDER = "folders/%%%folderid%%%/recycle?targetfolder=%%%targetfolderid%%%&sourceclient=%%%sourceclient%%%";
        public const string MOVE_FILE = "files/%%%fileid%%%/parent?targetfolder=%%%targetfolderid%%%&sourceclient=%%%sourceclient%%%";
        public const string MOVE_FOLDER = "folders/%%%folderid%%%/parent?targetfolder=%%%targetfolderid%%%&sourceclient=%%%sourceclient%%%";
        public const string GET_PROGRESS_KEY = "sites/%%%siteid%%%/documentprogresskey";
        public const string GET_PROGRESS_KEY_STATUS = "sites/%%%siteid%%%/documentstatus/%%%progresskey%%%?rootfolderid=%%%rootfolderid%%%";
        public const string GET_CHANGES_LIST = "changes?siteid=%%%siteid%%%&limit=%%%limit%%%&syncpositionid=%%%syncpositionid%%%";
        public const string ADD_FILE_FOR_QUICK_SHARE = "shares/content?progressiveoperkey=%%%progressiveoperkey%%%";
        public const string SHARE_VIA_LINK = "shares/link";

        public const string OAUTH_TOKEN_PART = "authorize.action?response_type=code&client_id=%%%clientid%%%";
        public const string OAUTH_TOKEN_REDIRECT_PART = "&redirect_uri=%%%redirecturi%%%";
        public const string OAUTH_PART = "oauth2/token";
        public const string OATH_BODY = "code=%%%token%%%&client_id=%%%client_id%%%&client_secret=%%%client_secret%%%&grant_type=authorization_code";  //<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>
        public const string OATH_REFRESH_ACCESS_KEY_BODY = "client_id=%%%client_id%%%&client_secret=%%%client_secret%%%&grant_type=refresh_token&refresh_token=%%%refresh_token%%%";  //<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>

        public const string ADDFOLDER_XML = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><folder><name>%%%name%%%</name><description>%%%description%%%</description><accessInherited>1</accessInherited></folder>";
        public const string UPDATEFOLDER_XML = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><folder><name>%%%name%%%</name><description>%%%description%%%</description></folder>";
        public const string UPDATEFILE_XML = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><file><filename>%%%name%%%</filename></file>";
        public const string SHAREVIALINK_XML = "<share><resourcetype>%%%folderorfile%%%</resourcetype><resourceid>%%%resourceid%%%</resourceid><permissions>%%%permissions%%%</permissions><shareoption>%%%shareoption%%%</shareoption><expiry>%%%expiry%%%</expiry></share>";
        #endregion

        #region Variable Declaration
        public enum FileName
        {
            OnlyExt = 1,
            WithoutExt = 2,
            WithExt = 3
        }

        private int readTimeOutMili = 600000; //10Min read timeout

        List<APIInfo> APIList = new List<APIInfo>();
        List<APIParameter> APIParameterList = new List<APIParameter>();
        #endregion

        #region General Messages
        private static string Message_0004 = "[(Function:%%%methodname%%%)%%%line#%%%]: ";
        public const string Message_0021 = "Error from execute API request. ";
        public const string Message_0075 = "Please enter API Url value.";
        public const string Message_0076 = "Please enter Access Key value.";
        public const string Message_0077 = "Please provide below listed parameter(s) value.";
        public const string Message_0079 = "File not exist at specified path.";
        #endregion

        #region Page Events
        private void frmAPIMethodsCall_Load(object sender, EventArgs e)
        {
            try
            {
                SetDefaultSettings();
                LoadAPIList();
                LoadAPIParameter();
                GetParameterFile();
                FillAPIMethods();
                LoadParameterValue();
                LoadUrlAndKey();
                ShowHideParameterControl();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Page Load Event");
            }
        }
        #endregion

        #region Control Events
        private void rbtnXML_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                ShowHideParameterControl();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "rbtnXML_CheckedChanged");
            }
        }

        private void rbtnAskForParameter_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                ShowHideParameterControl();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "rbtnAskForParameter_CheckedChanged");
            }
        }

        private void cmbAPIMethod_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                LoadParameterInGrid();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "cmbAPIMethod_SelectedIndexChanged");
            }
        }

        private void btnCallAPI_Click(object sender, EventArgs e)
        {
            string strAPIUrl = string.Empty;
            string strBody = string.Empty;
            string strFilePath = string.Empty;
            string strRequestType = string.Empty;
            try
            {
                ShowHideLoading(true);
                LoadParameterValue();
                WriteUrlAndKey();

                strAPIUrl = txtAPIUrl.Text.Trim().Trim('\\').Trim().Trim().Trim('/').Trim() + '/';
                txtOutputBody.Text = string.Empty;
                lblStatusResult.Text = string.Empty;
                txtAPICallList.Text = string.Empty;
                Application.DoEvents();

                var objAPIItem = (from item in APIList
                                  where item.APIID == Convert.ToInt32(cmbAPIMethod.SelectedValue.ToString())
                                  select item).FirstOrDefault();

                enmAPIList enmSelected = (enmAPIList)Enum.Parse(typeof(enmAPIList), cmbAPIMethod.SelectedValue.ToString());
                switch (enmSelected)
                {
                    case enmAPIList.OAUTH:
                        strAPIUrl = strAPIUrl.Substring(0, strAPIUrl.ToUpper().IndexOf("/API") + 4).Trim() + '/' + objAPIItem.APIUrl;
                        strBody = objAPIItem.APIBody;
                        strRequestType = objAPIItem.RequestType;

                        ReplaceValue(ref strAPIUrl, ref strBody, ref strFilePath);
                        if (!ValidateControl(strAPIUrl, strBody, strFilePath))
                            return;
                        ExecuteAPIRequest(strRequestType.ToUpper(), strAPIUrl, strBody, string.Empty, ref strFilePath);
                        break;
                    case enmAPIList.ADD_FILE:
                    case enmAPIList.ADD_FILE_VERSION:
                        strAPIUrl += objAPIItem.APIUrl;
                        strBody = objAPIItem.APIBody;
                        strRequestType = objAPIItem.RequestType;

                        ReplaceValue(ref strAPIUrl, ref strBody, ref strFilePath);
                        if (!ValidateControl(strAPIUrl, strBody, strFilePath, true, true))
                            return;
                        AddFile(strAPIUrl, txtAccessKey.Text.Trim(), Path.GetFileName(strFilePath), strFilePath);
                        break;
                    case enmAPIList.DOWNLOAD_FILE:
                        string strAPIUrl_F = strAPIUrl + GET_FILE_INFO;
                        ReplaceValue(ref strAPIUrl_F, ref strBody, ref strFilePath);
                        if (!ValidateControl(strAPIUrl_F, strBody, strFilePath, true))
                            return;
                        ExecuteAPIRequest(RequestType.Get.ToString(), strAPIUrl_F, strBody, txtAccessKey.Text.Trim(), ref strFilePath, true);
                        strFilePath = Path.Combine(Application.StartupPath, strFilePath);

                        strAPIUrl += objAPIItem.APIUrl;
                        strBody = objAPIItem.APIBody;
                        strRequestType = objAPIItem.RequestType;

                        ReplaceValue(ref strAPIUrl, ref strBody, ref strFilePath);
                        if (!ValidateControl(strAPIUrl, strBody, strFilePath, true))
                            return;
                        DownloadFile(strAPIUrl, txtAccessKey.Text.Trim(), strFilePath);
                        break;
                    default:
                        strAPIUrl += objAPIItem.APIUrl;
                        strBody = objAPIItem.APIBody;
                        strRequestType = objAPIItem.RequestType;

                        ReplaceValue(ref strAPIUrl, ref strBody, ref strFilePath);
                        if (!ValidateControl(strAPIUrl, strBody, strFilePath, true))
                            return;
                        ExecuteAPIRequest(strRequestType.ToUpper(), strAPIUrl, strBody, txtAccessKey.Text.Trim(), ref strFilePath);
                        break;
                }
            }
            catch
            {
                //MessageBox.Show(ex.Message.ToString(), "Call API button click event");
            }
            finally
            {
                ShowHideLoading(false);
                Application.DoEvents();
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            try
            {
                cmbAPIMethod.Focus();
                txtOutputBody.Text = string.Empty;
                lblStatusResult.Text = string.Empty;
                txtAPICallList.Text = string.Empty;
                foreach (DataGridViewRow item in gvParameters.Rows)
                {
                    item.Cells["ParameterValue"].Value = string.Empty;
                }
                for (int i = 0; i < APIParameterList.Count; i++)
                {
                    APIParameterList[i].ParameterValue = string.Empty;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Clear button click event");
            }
        }

        private void btnProxySetting_Click(object sender, EventArgs e)
        {
            try
            {
                ProxySettings(true);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "btnProxySetting_Click");
            }
        }

        private void btnSaveProxy_Click(object sender, EventArgs e)
        {
            try
            {
                ProxySettings(false);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "btnSaveProxy_Click");
            }
        }

        private void btnProxyClear_Click(object sender, EventArgs e)
        {
            try
            {
                txtProxyServer.Text = string.Empty;
                txtPort.Text = string.Empty;
                txtUserName.Text = string.Empty;
                txtPwd.Text = string.Empty;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "btnProxyClear_Click");
            }
        }
        #endregion

        #region Methods
        private void SetDefaultSettings()
        {
            txtOutputBody.Text = string.Empty;
            lblStatusResult.Text = string.Empty;
            txtAPICallList.Text = string.Empty;
            gvParameters.Enabled = false;
            pnlProxySettings.Visible = false;
            lblWait.Visible = false;
        }

        private void ShowHideLoading(bool isShowLoading)
        {
            lblWait.Visible = isShowLoading;
            gpAPICall.Enabled = !isShowLoading;
        }

        private void LoadAPIList()
        {
            try
            {
                APIList.Add(new APIInfo() { APIID = 1, APIUrl = GET_SITE_LIST, APIBody = string.Empty, APIDisplayName = "Get Site List", RequestType = "GET" });
                APIList.Add(new APIInfo() { APIID = 2, APIUrl = GET_FILE_INFO, APIBody = string.Empty, APIDisplayName = "Get File Info", RequestType = "GET" });
                APIList.Add(new APIInfo() { APIID = 3, APIUrl = GET_FOLDER_INFO, APIBody = string.Empty, APIDisplayName = "Get Folder Info", RequestType = "GET" });
                APIList.Add(new APIInfo() { APIID = 4, APIUrl = GET_FOLDERS_LIST, APIBody = string.Empty, APIDisplayName = "Get Folders List", RequestType = "GET" });
                APIList.Add(new APIInfo() { APIID = 5, APIUrl = GET_FILES_LIST, APIBody = string.Empty, APIDisplayName = "Get Files List", RequestType = "GET" });
                APIList.Add(new APIInfo() { APIID = 6, APIUrl = ADD_FOLDER, APIBody = ADDFOLDER_XML, APIDisplayName = "Add Folder", RequestType = "POST" });
                APIList.Add(new APIInfo() { APIID = 7, APIUrl = UPDATE_FOLDER, APIBody = UPDATEFOLDER_XML, APIDisplayName = "Update Folder", RequestType = "PUT" });
                APIList.Add(new APIInfo() { APIID = 8, APIUrl = DELETE_FOLDER, APIBody = string.Empty, APIDisplayName = "Delete Folder", RequestType = "DELETE" });
                APIList.Add(new APIInfo() { APIID = 9, APIUrl = ADD_FILE, APIBody = string.Empty, APIDisplayName = "Add File", RequestType = "POST" }); //Seperate
                APIList.Add(new APIInfo() { APIID = 10, APIUrl = ADD_FILE_VERSION, APIBody = string.Empty, APIDisplayName = "Add File Version", RequestType = "POST" }); //Seperate
                APIList.Add(new APIInfo() { APIID = 11, APIUrl = UPDATE_FILE, APIBody = UPDATEFILE_XML, APIDisplayName = "Update File", RequestType = "PUT" });
                APIList.Add(new APIInfo() { APIID = 12, APIUrl = DELETE_FILE, APIBody = string.Empty, APIDisplayName = "Delete File", RequestType = "DELETE" });
                APIList.Add(new APIInfo() { APIID = 13, APIUrl = DOWNLOAD_FILE, APIBody = string.Empty, APIDisplayName = "Download File", RequestType = "" });  //Seperate
                APIList.Add(new APIInfo() { APIID = 14, APIUrl = RESTORE_FILE, APIBody = string.Empty, APIDisplayName = "Restore File", RequestType = "POST" });
                APIList.Add(new APIInfo() { APIID = 15, APIUrl = RESTORE_FOLDER, APIBody = string.Empty, APIDisplayName = "Restore Folder", RequestType = "POST" });
                APIList.Add(new APIInfo() { APIID = 16, APIUrl = MOVE_FILE, APIBody = string.Empty, APIDisplayName = "Move File", RequestType = "PUT" });
                APIList.Add(new APIInfo() { APIID = 17, APIUrl = MOVE_FOLDER, APIBody = string.Empty, APIDisplayName = "Move Folder", RequestType = "PUT" });
                APIList.Add(new APIInfo() { APIID = 18, APIUrl = GET_PROGRESS_KEY, APIBody = string.Empty, APIDisplayName = "Get Progress Key", RequestType = "GET" });
                APIList.Add(new APIInfo() { APIID = 19, APIUrl = GET_PROGRESS_KEY_STATUS, APIBody = string.Empty, APIDisplayName = "Get Progress Key Status", RequestType = "POST" });
                APIList.Add(new APIInfo() { APIID = 20, APIUrl = GET_CHANGES_LIST, APIBody = string.Empty, APIDisplayName = "Get Change List", RequestType = "GET" });
                APIList.Add(new APIInfo() { APIID = 21, APIUrl = OAUTH_PART, APIBody = OATH_BODY, APIDisplayName = "OAuth (Generate Access Token)", RequestType = "POST" });
                //APIList.Add(new APIInfo() { APIID = 22, APIUrl = ADD_FILE_FOR_QUICK_SHARE, APIBody = string.Empty, APIDisplayName = "Add File For Quick Share", RequestType = "POST" });
                //APIList.Add(new APIInfo() { APIID = 23, APIUrl = SHARE_VIA_LINK, APIBody = SHAREVIALINK_XML, APIDisplayName = "Share Via Link", RequestType = "POST" });
                
                //APIList.Add(new APIInfo() { APIID = 25, APIUrl = "", APIDisplayName = "", RequestType = "" });
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private void LoadAPIParameter()
        {
            try
            {
                APIParameterList.Add(new APIParameter() { APIParameterID = 1, ParameterName = "%%%siteid%%%", ParameterDisplayName = "Site Id", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 2, ParameterName = "%%%client_id%%%", ParameterDisplayName = "Client Id", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 3, ParameterName = "%%%client_secret%%%", ParameterDisplayName = "Client Secret", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 4, ParameterName = "%%%clientid%%%", ParameterDisplayName = "Client Id", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 5, ParameterName = "%%%token%%%", ParameterDisplayName = "Token", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 6, ParameterName = "%%%redirect_uri%%%", ParameterDisplayName = "Redirect Uri", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 7, ParameterName = "%%%refresh_token%%%", ParameterDisplayName = "Refresh Token", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 8, ParameterName = "%%%progresskey%%%", ParameterDisplayName = "Progress Key", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 9, ParameterName = "%%%limit%%%", ParameterDisplayName = "Limit", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 10, ParameterName = "%%%syncpositionid%%%", ParameterDisplayName = "Sync Position Id", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 11, ParameterName = "%%%name%%%", ParameterDisplayName = "Name", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 12, ParameterName = "%%%description%%%", ParameterDisplayName = "Description", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 13, ParameterName = "%%%objectid%%%", ParameterDisplayName = "Object Id", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 14, ParameterName = "%%%fileid%%%", ParameterDisplayName = "File Id", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 15, ParameterName = "%%%folderid%%%", ParameterDisplayName = "Folder Id", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 16, ParameterName = "%%%rootfolderid%%%", ParameterDisplayName = "Root Folder Id", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 17, ParameterName = "%%%parentfolderid%%%", ParameterDisplayName = "Parent Folder Id", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 18, ParameterName = "%%%sourceclient%%%", ParameterDisplayName = "Source Client", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 19, ParameterName = "%%%targetfolderid%%%", ParameterDisplayName = "Target Folder Id", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 20, ParameterName = "%%%filepath%%%", ParameterDisplayName = "File Path", HelpText = string.Empty });

                APIParameterList.Add(new APIParameter() { APIParameterID = 21, ParameterName = "%%%progressiveoperkey%%%", ParameterDisplayName = "Progress Key", HelpText = string.Empty });
                APIParameterList.Add(new APIParameter() { APIParameterID = 22, ParameterName = "%%%folderorfile%%%", ParameterDisplayName = "Folder or File", HelpText = "Just type 'folder' or 'file'" });
                APIParameterList.Add(new APIParameter() { APIParameterID = 23, ParameterName = "%%%resourceid%%%", ParameterDisplayName = "Resource Id", HelpText = "The id can be any from my files or shared items" });
                APIParameterList.Add(new APIParameter() { APIParameterID = 24, ParameterName = "%%%permissions%%%", ParameterDisplayName = "Permissions", HelpText = "Integer value for the two options : 1(download_view_print) / 2(view_only)" });
                APIParameterList.Add(new APIParameter() { APIParameterID = 25, ParameterName = "%%%shareoption%%%", ParameterDisplayName = "Share Option", HelpText = "Integer value for the options : 1(anyone) / 2(my organisation user) / 3(system users)" });
                APIParameterList.Add(new APIParameter() { APIParameterID = 26, ParameterName = "%%%expiry%%%", ParameterDisplayName = "Expiry", HelpText = "UTC date and time(dd MMM yyyy HH:mm)" });
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private void ShowHideParameterControl()
        {
            try
            {
                LoadParameterInGrid();
                gvParameters.Enabled = rbtnAskForParameter.Checked;

                if (!gvParameters.Enabled)
                {
                    gvParameters.DefaultCellStyle.BackColor = SystemColors.Control;
                    gvParameters.DefaultCellStyle.ForeColor = SystemColors.GrayText;
                    gvParameters.ColumnHeadersDefaultCellStyle.BackColor = SystemColors.Control;
                    gvParameters.ColumnHeadersDefaultCellStyle.ForeColor = SystemColors.GrayText;
                    gvParameters.EnableHeadersVisualStyles = false;
                }
                else
                {
                    gvParameters.DefaultCellStyle.BackColor = SystemColors.Window;
                    gvParameters.DefaultCellStyle.ForeColor = SystemColors.ControlText;
                    gvParameters.ColumnHeadersDefaultCellStyle.BackColor = SystemColors.Window;
                    gvParameters.ColumnHeadersDefaultCellStyle.ForeColor = SystemColors.ControlText;
                    gvParameters.EnableHeadersVisualStyles = true;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private void FillAPIMethods()
        {
            try
            {
                cmbAPIMethod.DataSource = APIList.OrderBy(a => a.APIDisplayName).ToList();
                cmbAPIMethod.DisplayMember = "APIDisplayName";
                cmbAPIMethod.ValueMember = "APIID";
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private bool ValidateControl(string strAPIUrl, string strBody, string strFilePath, bool blnRequiredAccess = false, bool blnFileRelated = false)
        {
            string strErrorMessage = string.Empty;
            try
            {
                if (txtAPIUrl.Text.Trim() == string.Empty)
                    strErrorMessage += Message_0075 + System.Environment.NewLine;
                if (blnRequiredAccess && txtAccessKey.Text.Trim() == string.Empty)
                    strErrorMessage += Message_0076 + System.Environment.NewLine;
                if (strAPIUrl.Contains("%%%") || strBody.Contains("%%%") || (blnFileRelated && strFilePath == string.Empty))
                    strErrorMessage += Message_0077 + System.Environment.NewLine;
                if (strAPIUrl.Contains("%%%"))
                {
                    foreach (string item in GetParameter(strAPIUrl))
                    {
                        var objAPIParameter = APIParameterList.Where(P => P.ParameterName == item).FirstOrDefault();
                        if (objAPIParameter != null)
                            strErrorMessage += " -> " + objAPIParameter.ParameterDisplayName + System.Environment.NewLine;
                    }
                }
                if (strBody.Contains("%%%"))
                {
                    foreach (string item in GetParameter(strBody))
                    {
                        var objAPIParameter = APIParameterList.Where(P => P.ParameterName == item).FirstOrDefault();
                        if (objAPIParameter != null)
                            strErrorMessage += " -> " + objAPIParameter.ParameterDisplayName + System.Environment.NewLine;
                    }
                }
                if (blnFileRelated && strFilePath == string.Empty)
                    strErrorMessage += " -> File Path" + System.Environment.NewLine;
                if (blnFileRelated && strFilePath != string.Empty && !File.Exists(strFilePath))
                    strErrorMessage += Message_0079 + System.Environment.NewLine;

                if (strErrorMessage != string.Empty)
                {
                    MessageBox.Show(strErrorMessage, "Required Field Validator");
                    return false;
                }
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private void ReplaceValue(ref string strAPIUrl, ref string strBody, ref string strFilePath)
        {
            try
            {
                for (int i = 0; i < APIParameterList.Count; i++)
                {
                    if (APIParameterList[i].ParameterValue != null && APIParameterList[i].ParameterValue.Trim() != string.Empty)
                    {
                        strAPIUrl = strAPIUrl.Replace(APIParameterList[i].ParameterName, APIParameterList[i].ParameterValue);
                        strBody = strBody.Replace(APIParameterList[i].ParameterName, APIParameterList[i].ParameterValue);
                        if (strFilePath.Length <= 0 && APIParameterList[i].ParameterName == "%%%filepath%%%")
                            strFilePath = ("%%%filepath%%%").Replace(APIParameterList[i].ParameterName, APIParameterList[i].ParameterValue);
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private void LoadParameterValue()
        {
            try
            {
                if (rbtnXML.Checked)
                {
                    XmlDocument xmlDoc = new XmlDocument();
                    xmlDoc.Load(GetParameterFile());
                    XmlNode nodeParameters = xmlDoc.DocumentElement.SelectSingleNode("/Parameters");
                    foreach (XmlNode nodePar in nodeParameters)
                    {
                        for (int i = 0; i < APIParameterList.Count; i++)
                        {
                            if (nodePar.Name.Trim().ToLower() == APIParameterList[i].ParameterDisplayName.Trim().Replace(" ", "").ToLower() && nodePar.InnerText.Trim() != string.Empty)
                                APIParameterList[i].ParameterValue = nodePar.InnerText.Trim();
                        }
                    }
                }
                else
                {
                    foreach (DataGridViewRow gvRowItem in gvParameters.Rows)
                    {
                        for (int i = 0; i < APIParameterList.Count; i++)
                        {
                            if (Convert.ToInt16(gvRowItem.Cells["APIParameterId"].Value) == APIParameterList[i].APIParameterID && Convert.ToString(gvRowItem.Cells["ParameterValue"].Value).Trim().Length > 0)
                                APIParameterList[i].ParameterValue = Convert.ToString(gvRowItem.Cells["ParameterValue"].Value).Trim();
                        }
                    }

                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private string GetParameterFile()
        {
            string strParameterFile = string.Empty;
            try
            {
                strParameterFile = Path.Combine(Application.StartupPath, "Parameters.xml");
                if (!File.Exists(strParameterFile))
                {
                    XmlDocument xmlDoc = new XmlDocument();
                    XmlElement rootNode = xmlDoc.CreateElement("Parameters");
                    xmlDoc.AppendChild(rootNode);
                    for (int i = 0; i < APIParameterList.Count; i++)
                    {
                        AddNode(xmlDoc, rootNode, APIParameterList[i].ParameterDisplayName.Trim().Replace(" ", ""), string.Empty);
                    }
                    AddNode(xmlDoc, rootNode, "APIUrl", txtAPIUrl.Text.Trim());
                    AddNode(xmlDoc, rootNode, "AccessKey", txtAccessKey.Text.Trim());

                    xmlDoc.Save(strParameterFile);
                }
                return strParameterFile;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private void LoadUrlAndKey()
        {
            try
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(GetParameterFile());
                XmlNode nodeAPIUrl = xmlDoc.DocumentElement.SelectSingleNode("/Parameters/APIUrl");
                if (nodeAPIUrl != null)
                    txtAPIUrl.Text = nodeAPIUrl.InnerText;
                XmlNode nodeAccessKey = xmlDoc.DocumentElement.SelectSingleNode("/Parameters/AccessKey");
                if (nodeAccessKey != null)
                    txtAccessKey.Text = nodeAccessKey.InnerText;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private void WriteUrlAndKey()
        {
            string strParameterFile = string.Empty;
            try
            {
                strParameterFile = GetParameterFile();
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(strParameterFile);
                XmlElement rootParameters = xmlDoc.DocumentElement;
                if (rootParameters == null)
                    return;

                if (txtAPIUrl.Text.Trim().Length > 0)
                    AddNode(xmlDoc, rootParameters, "APIUrl", txtAPIUrl.Text.Trim());
                if (txtAccessKey.Text.Trim().Length > 0)
                    AddNode(xmlDoc, rootParameters, "AccessKey", txtAccessKey.Text.Trim());

                xmlDoc.Save(strParameterFile);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private void AddNode(XmlDocument xmlDoc, XmlElement rootNode, string strNodeName, string strValue)
        {
            try
            {
                XmlNode nodeAPIUrl = rootNode.SelectSingleNode(strNodeName);
                if (nodeAPIUrl == null)
                {
                    XmlElement elem = xmlDoc.CreateElement(strNodeName);
                    if (strValue.Length > 0)
                        elem.InnerText = strValue;
                    rootNode.AppendChild(elem);
                }
                else if (strValue.Length > 0)
                    nodeAPIUrl.InnerText = strValue;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private void ProxySettings(bool isEnableProxy)
        {
            try
            {
                gpAPICall.Enabled = !isEnableProxy;
                tbAPI.Enabled = !isEnableProxy;
                pnlProxySettings.Visible = isEnableProxy;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public WebProxy GetProxySettings()
        {
            NetworkCredential objCred = null;
            WebProxy objProxy = null;
            try
            {
                if (txtProxyServer.Text.Trim().Length > 0)
                {
                    if (txtUserName.Text.Trim().Length > 0 || txtPwd.Text.Trim().Length > 0)
                    {
                        objCred = new NetworkCredential(txtUserName.Text.Trim(), txtPwd.Text.Trim());
                    }

                    if (txtPort.Text.Trim().Length > 0)
                        objProxy = new WebProxy(txtProxyServer.Text.Trim(), Convert.ToInt16(txtPort.Text.Trim()));
                    else
                        objProxy = new WebProxy(txtProxyServer.Text.Trim());

                    objProxy.BypassProxyOnLocal = false;
                    if (objCred != null)
                        objProxy.Credentials = objCred;
                }

                return objProxy;
            }
            catch (Exception ex)
            {
                throw GetExceptionMsg("GetProxySettings", ex);
            }
            finally
            {
                if (objCred != null)
                    objCred = null;

                if (objProxy != null)
                    objProxy = null;
            }
        }

        private void LoadParameterInGrid()
        {
            try
            {
                gvParameters.Rows.Clear();
                if (rbtnXML.Checked)
                    return;

                var objAPIList = APIList.Where(P => P.APIID == Convert.ToInt16(cmbAPIMethod.SelectedValue)).FirstOrDefault();
                if (objAPIList == null)
                    return;

                foreach (string item in GetParameter(objAPIList.APIUrl))
                {
                    if (item.Length <= 0)
                        continue;
                    var objAPIParameter = APIParameterList.Where(P => P.ParameterName == item).FirstOrDefault();
                    if (objAPIParameter == null)
                        continue;
                    AddParameterRow(objAPIParameter.APIParameterID, objAPIParameter.ParameterDisplayName, objAPIParameter.ParameterValue);
                }
                foreach (string item in GetParameter(objAPIList.APIBody))
                {
                    if (item.Length <= 0)
                        continue;
                    var objAPIParameter = APIParameterList.Where(P => P.ParameterName == item).FirstOrDefault();
                    if (objAPIParameter == null)
                        continue;
                    AddParameterRow(objAPIParameter.APIParameterID, objAPIParameter.ParameterDisplayName, objAPIParameter.ParameterValue);
                }

                enmAPIList enmSelected = (enmAPIList)Enum.Parse(typeof(enmAPIList), cmbAPIMethod.SelectedValue.ToString());
                switch (enmSelected)
                {
                    case enmAPIList.ADD_FILE:
                    case enmAPIList.ADD_FILE_VERSION:
                    //case enmAPIList.QUICK_SHARE:
                        var objAPIParameter = APIParameterList.Where(P => P.ParameterName == "%%%filepath%%%").FirstOrDefault();
                        if (objAPIParameter != null)
                            AddParameterRow(objAPIParameter.APIParameterID, objAPIParameter.ParameterDisplayName, objAPIParameter.ParameterValue);
                        break;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private string[] GetParameter(string strValue)
        {
            string strSubstring = string.Empty;
            string strParameter = string.Empty;
            try
            {
                while (strValue.Contains("%%%"))
                {
                    strSubstring = strValue.Substring(strValue.IndexOf("%%%") + 3);
                    strParameter += "%%%" + strSubstring.Substring(0, strSubstring.IndexOf("%%%") + 3) + "|";
                    strValue = strSubstring.Substring(strSubstring.IndexOf("%%%") + 3);
                }
                return strParameter.Trim('|').Split('|');
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private void AddParameterRow(int intParameterId, string strParDisplayName, string strParValue)
        {
            gvParameters.Rows.Add();
            int intRowIndex = gvParameters.RowCount - 1;
            DataGridViewRow gvrItem = gvParameters.Rows[intRowIndex];
            gvrItem.Cells["APIParameterId"].Value = intParameterId;
            gvrItem.Cells["ParameterName"].Value = strParDisplayName;
            gvrItem.Cells["ParameterValue"].Value = strParValue;
        }

        #endregion

        #region Methods for API Calls
        private bool ExecuteAPIRequest(string strMethod, string strUri, string strBody, string strAccessKey, ref string strReturnValue, bool isGetFileName = false)
        {
            HttpWebRequest httpWebRequest = null;
            HttpWebResponse response = null;
            byte[] buf = null;
            string strAPICall = string.Empty;
            string strNewLine = "\r\n";
            StreamReader sr = null;
            string responseString = string.Empty;
            byte[] byteArray = null;
            XmlDocument xmlDoc = null;
            StringBuilder sb = null;
            XmlWriterSettings settings = null;
            XmlWriter writer = null;
            try
            {
                strAPICall += "HttpWebRequest httpWebRequest = null; " + strNewLine;
                strAPICall += "HttpWebResponse response = null; " + strNewLine;
                strAPICall += "byte[] buf = null; " + strNewLine;
                strAPICall += strNewLine;

                httpWebRequest = (HttpWebRequest)WebRequest.Create(strUri);
                strAPICall += "httpWebRequest = (HttpWebRequest)WebRequest.Create(\"" + strUri + "\"); " + strNewLine;
                httpWebRequest.Method = strMethod;
                strAPICall += "httpWebRequest.Method = \"" + strMethod + "\"; " + strNewLine;
                if (txtProxyServer.Text.Trim().Length <= 0)
                    httpWebRequest.Credentials = CredentialCache.DefaultCredentials;
                else
                    httpWebRequest.Proxy = GetProxySettings();
                strAPICall += "httpWebRequest.Credentials = CredentialCache.DefaultCredentials; " + strNewLine;
                if (strAccessKey.Length > 0)
                {
                    httpWebRequest.Headers.Add(strAuthorizationKeyName, strBearer + strAccessKey);
                    strAPICall += "httpWebRequest.Headers.Add(\"" + strAuthorizationKeyName + "\", \"" + strBearer + strAccessKey + "\"); " + strNewLine;
                }

                if (strAccessKey == string.Empty)
                {
                    httpWebRequest.ContentType = strUrlEncoded;
                    strAPICall += "httpWebRequest.ContentType = \"" + strUrlEncoded + "\"; " + strNewLine;
                }
                else
                {
                    httpWebRequest.ContentType = strXMLContentType;
                    strAPICall += "httpWebRequest.ContentType = \"" + strXMLContentType + "\"; " + strNewLine;
                    httpWebRequest.Accept = strXMLContentType;
                    strAPICall += "httpWebRequest.Accept = \"" + strXMLContentType + "\"; " + strNewLine;
                }
                //httpWebRequest.Timeout = connectionTimeOutMili;
                //strAPICall += "httpWebRequest.Timeout = " + connectionTimeOutMili + "; " + strNewLine;
                httpWebRequest.ReadWriteTimeout = readTimeOutMili;
                strAPICall += "httpWebRequest.ReadWriteTimeout = " + readTimeOutMili + "; " + strNewLine;
                //httpWebRequest.ContinueTimeout = connectionTimeOutMili;
                //strAPICall += "httpWebRequest.ContinueTimeout = " + connectionTimeOutMili + "; " + strNewLine;
                //httpWebRequest.AllowWriteStreamBuffering = false;  //New
                //strAPICall += "httpWebRequest.AllowWriteStreamBuffering = false; " + strNewLine;
                //httpWebRequest.AllowReadStreamBuffering = false;  //New
                //strAPICall += "httpWebRequest.AllowReadStreamBuffering = false; " + strNewLine;

                if (strBody.Length > 0)
                {
                    strAPICall += strNewLine;
                    buf = Encoding.UTF8.GetBytes(strBody);
                    strAPICall += "buf = Encoding.UTF8.GetBytes(\"" + strBody.Replace("\"", "\\\"") + "\"); " + strNewLine;
                    httpWebRequest.ContentLength = buf.Length;
                    strAPICall += "httpWebRequest.ContentLength = buf.Length; " + strNewLine;
                    httpWebRequest.GetRequestStream().Write(buf, 0, buf.Length);
                    strAPICall += "httpWebRequest.GetRequestStream().Write(buf, 0, buf.Length); " + strNewLine;
                }
                else
                {
                    httpWebRequest.ContentLength = 0;
                    strAPICall += "httpWebRequest.ContentLength = 0; " + strNewLine;
                }
                strAPICall += strNewLine;

                response = (HttpWebResponse)httpWebRequest.GetResponse();   //Working fine in Sync mode.
                strAPICall += "response = (HttpWebResponse)httpWebRequest.GetResponse(); " + strNewLine;
                if (buf != null)
                {
                    buf = null;
                    strAPICall += "buf = null; " + strNewLine;
                    strAPICall += strNewLine;
                }

                if (!isGetFileName)
                    lblStatusResult.Text = (int)response.StatusCode + " " + response.StatusCode;  //for our use
                if (response.StatusCode == HttpStatusCode.OK || response.StatusCode == HttpStatusCode.Created || response.StatusCode == HttpStatusCode.NoContent)
                {
                    strAPICall += strNewLine;
                    strAPICall += "if(response.StatusCode == HttpStatusCode.OK || response.StatusCode == HttpStatusCode.Created || response.StatusCode == HttpStatusCode.NoContent) " + strNewLine;
                    strAPICall += "{ " + strNewLine;
                    strAPICall += "\r\tStreamReader sr = new StreamReader(response.GetResponseStream()); " + strNewLine;
                    strAPICall += "\r\tstring strResponse = sr.ReadToEnd(); " + strNewLine;
                    strAPICall += "} " + strNewLine;
                    if (!isGetFileName)
                        txtAPICallList.Text = strAPICall;

                    sr = new StreamReader(response.GetResponseStream(), Encoding.UTF8);
                    responseString = sr.ReadToEnd();
                    if (responseString.Length > 0 && responseString.Contains("<?xml"))
                    {
                        // convert string to stream
                        byteArray = Encoding.ASCII.GetBytes(responseString);
                        MemoryStream stream = new MemoryStream(byteArray);

                        xmlDoc = new XmlDocument();
                        xmlDoc.Load(stream);
                        if (isGetFileName)
                        {
                            XmlNode nodeParameters = xmlDoc.DocumentElement.SelectSingleNode("/file");
                            foreach (XmlNode nodePar in nodeParameters)
                            {
                                if (nodePar.Name.Trim() == "name")
                                    strReturnValue = nodePar.InnerText.Trim();
                                if (nodePar.Name.Trim() == "type")
                                    strReturnValue += "." + nodePar.InnerText.Trim();
                            }
                        }
                        else
                        {
                            sb = new StringBuilder();
                            settings = new XmlWriterSettings();
                            settings.Indent = true;
                            settings.IndentChars = "  ";
                            settings.NewLineChars = "\r\n";
                            settings.NewLineHandling = NewLineHandling.Replace;
                            writer = XmlWriter.Create(sb, settings);
                            xmlDoc.WriteTo(writer);
                            writer.Close();
                            txtOutputBody.Text = sb.ToString();
                        }
                    }
                    else
                        txtOutputBody.Text = responseString;
                    return true;
                }
                else
                    return false;
            }
            catch (Exception ex)
            {
                txtOutputBody.Text = ex.Message;
                throw GetExceptionMsg("ExecuteAPIRequest", new Exception(Message_0021 + ex.Message));
            }
            finally
            {
                response = null;
                httpWebRequest = null;
                sr = null;
                byteArray = null;
                xmlDoc = null;
                sb = null;
                settings = null;
                writer = null;
            }
        }

        public bool AddFile(string strAPIURL, string strAccessKey, string strFileName, string strFilePath)
        {
            string lineEnd = "\r\n"; //(char)10 & (char)13
            string twoHyphens = "--";
            string boundary = "AaB03x";

            byte[] buffer;
            Stream requestStream = null;
            HttpWebRequest httpWebRequest = null;
            HttpWebResponse response = null;
            FileStream fileStream = null;

            string strAPICall = string.Empty;
            string strNewLine = "\r\n";
            XmlDocument xmlDoc = null;
            StringBuilder sb = null;
            XmlWriterSettings settings = null;
            XmlWriter writer = null;
            try
            {
                strAPICall += "string lineEnd = \"\\r\\n\"; " + strNewLine;
                strAPICall += "string twoHyphens = \"--\"; " + strNewLine;
                strAPICall += "string boundary = \"AaB03x\"; " + strNewLine;
                strAPICall += strNewLine;
                strAPICall += "byte[] buffer; " + strNewLine;
                strAPICall += "Stream requestStream = null; " + strNewLine;
                strAPICall += "HttpWebRequest httpWebRequest = null; " + strNewLine;
                strAPICall += "HttpWebResponse response = null; " + strNewLine;
                strAPICall += "FileStream fileStream = null; " + strNewLine;
                strAPICall += strNewLine;

                string line1 = twoHyphens + boundary + lineEnd;
                strAPICall += "string line1 = twoHyphens + boundary + lineEnd; " + strNewLine;
                string line2 = "Content-Disposition: form-data; name=\"file\";" + " filename=\"" + strFileName + "\"" + lineEnd; //Original text //string line2 = "Content-Disposition: form-data; name=\"file\";" + " filename=\"" + EncodeString(strFileName) + "\"" + lineEnd;
                strAPICall += "string line2 = \"Content-Disposition: form-data; name=\\\"file\\\";\" + \" filename=\\\"" + strFileName + "\\\"\" + lineEnd; " + strNewLine;
                string line3 = lineEnd;
                strAPICall += "string line3 = lineEnd; " + strNewLine;
                string line4 = twoHyphens + boundary + twoHyphens + lineEnd;
                strAPICall += "string line4 = twoHyphens + boundary + twoHyphens + lineEnd; " + strNewLine;
                string line5 = ("Content-Type: content/stream;" + lineEnd);
                strAPICall += "string line5 = (\"Content-Type: content/stream;\" + lineEnd); " + strNewLine;
                string line6 = "Content-Disposition: form-data; name=\"filename\"" + lineEnd + lineEnd + "" + GetFileName(strFileName, FileName.WithoutExt) + "";
                strAPICall += "string line6 = \"Content-Disposition: form-data; name=\\\"filename\\\"\" + lineEnd + lineEnd + \"\" + \"" + strFileName + "\" + \"\"; " + strNewLine;
                string line7 = "Content-Disposition: form-data; name=\"progressiveoperkey\"" + lineEnd + lineEnd;
                strAPICall += "string line7 = \"Content-Disposition: form-data; name=\\\"progressiveoperkey\\\"\" + lineEnd + lineEnd; " + strNewLine;
                strAPICall += strNewLine;

                //httpWebRequest = (HttpWebRequest)WebRequest.Create(strAPIURL + ADD_FILE.Replace("%%%rootfolderid%%%", intRootFolderID.ToString()).Replace("%%%parentfolderid%%%", intParentID.ToString()).Replace("%%%sourceclient%%%", GetMACID()));
                httpWebRequest = (HttpWebRequest)WebRequest.Create(strAPIURL);
                strAPICall += "httpWebRequest = (HttpWebRequest)WebRequest.Create(@\"" + strAPIURL + "\"); " + strNewLine;
                httpWebRequest.ContentType = "multipart/form-data;boundary=" + boundary;
                strAPICall += "httpWebRequest.ContentType = \"multipart/form-data;boundary=\" + boundary; " + strNewLine;
                httpWebRequest.Method = RequestType.Post.ToString();
                strAPICall += "httpWebRequest.Method = \"POST\"; " + strNewLine;
                if (txtProxyServer.Text.Trim().Length <= 0)
                    httpWebRequest.Credentials = CredentialCache.DefaultCredentials;
                else
                    httpWebRequest.Proxy = GetProxySettings();
                strAPICall += "httpWebRequest.Credentials = CredentialCache.DefaultCredentials; " + strNewLine;
                httpWebRequest.Headers.Add(strAuthorizationKeyName, strBearer + strAccessKey);
                strAPICall += "httpWebRequest.Headers.Add(\"" + strAuthorizationKeyName + "\",\"" + strBearer + strAccessKey + "\"); " + strNewLine;
                httpWebRequest.Accept = strXMLContentType;
                strAPICall += "httpWebRequest.Accept = \"" + strXMLContentType + "\"; " + strNewLine;
                //httpWebRequest.Timeout = connectionTimeOutMili;
                //strAPICall += "httpWebRequest.Timeout = " + connectionTimeOutMili + "; " + strNewLine;
                httpWebRequest.ReadWriteTimeout = readTimeOutMili;
                strAPICall += "httpWebRequest.ReadWriteTimeout = " + readTimeOutMili + "; " + strNewLine;
                //httpWebRequest.ContinueTimeout = connectionTimeOutMili;
                //strAPICall += "httpWebRequest.ContinueTimeout = " + connectionTimeOutMili + "; " + strNewLine;
                httpWebRequest.AllowWriteStreamBuffering = false;  //New
                strAPICall += "httpWebRequest.AllowWriteStreamBuffering = false; " + strNewLine;
                //httpWebRequest.AllowReadStreamBuffering = false;  //New
                //strAPICall += "httpWebRequest.AllowReadStreamBuffering = false; " + strNewLine;
                httpWebRequest.KeepAlive = true;
                strAPICall += "httpWebRequest.KeepAlive = true; " + strNewLine;
                //httpWebRequest.SendChunked = true; //This will not work in femework 3.5
                //strAPICall += "httpWebRequest.SendChunked = true; " + strNewLine;
                strAPICall += strNewLine;

                string postHeader = line1 + line2 + line5 + line3;
                strAPICall += "string postHeader = line1 + line2 + line5 + line3; " + strNewLine;
                byte[] postHeaderBytes = Encoding.UTF8.GetBytes(postHeader);
                strAPICall += "byte[] postHeaderBytes = Encoding.UTF8.GetBytes(postHeader); " + strNewLine;
                byte[] boundaryBytes = Encoding.ASCII.GetBytes(line3 + line1 + line6 + line3 + line4); //xxxxx
                strAPICall += "byte[] boundaryBytes = Encoding.ASCII.GetBytes(line3 + line1 + line6 + line3 + line4); " + strNewLine;
                strAPICall += strNewLine;

                fileStream = new FileStream(strFilePath, FileMode.Open, FileAccess.Read);
                strAPICall += "fileStream = new FileStream(@\"" + strFilePath + "\", FileMode.Open, FileAccess.Read); " + strNewLine;
                long length = postHeaderBytes.Length + fileStream.Length + boundaryBytes.Length;
                strAPICall += "long length = postHeaderBytes.Length + fileStream.Length + boundaryBytes.Length; " + strNewLine;
                httpWebRequest.ContentLength = length;
                strAPICall += "httpWebRequest.ContentLength = length; " + strNewLine;
                strAPICall += strNewLine;

                requestStream = httpWebRequest.GetRequestStream();
                strAPICall += "requestStream = httpWebRequest.GetRequestStream(); " + strNewLine;
                strAPICall += strNewLine;

                // Write out our post header
                requestStream.Write(postHeaderBytes, 0, postHeaderBytes.Length);
                strAPICall += "requestStream.Write(postHeaderBytes, 0, postHeaderBytes.Length); " + strNewLine;
                strAPICall += strNewLine;

                // Write out the file contents
                buffer = new Byte[checked((long)Math.Min(1024 * 512, (long)fileStream.Length))];
                strAPICall += "buffer = new Byte[checked((long)Math.Min(1024 * 512, (long)fileStream.Length))]; " + strNewLine;
                int bytesRead = 0;
                strAPICall += "int bytesRead = 0; " + strNewLine;
                while ((bytesRead = fileStream.Read(buffer, 0, buffer.Length)) != 0)
                    requestStream.Write(buffer, 0, bytesRead);
                strAPICall += "while ((bytesRead = fileStream.Read(buffer, 0, buffer.Length)) != 0) " + strNewLine;
                strAPICall += "requestStream.Write(buffer, 0, bytesRead); " + strNewLine;
                strAPICall += strNewLine;

                // Write out the trailing boundary
                requestStream.Write(boundaryBytes, 0, boundaryBytes.Length);
                strAPICall += "requestStream.Write(boundaryBytes, 0, boundaryBytes.Length); " + strNewLine;
                strAPICall += strNewLine;

                strAPICall += " " + strNewLine;
                response = (HttpWebResponse)httpWebRequest.GetResponse();
                strAPICall += "response = (HttpWebResponse)httpWebRequest.GetResponse(); " + strNewLine;
                if (buffer != null)
                {
                    buffer = null;
                    strAPICall += "buffer = null; " + strNewLine;
                    strAPICall += strNewLine;
                }

                lblStatusResult.Text = (int)response.StatusCode + " " + response.StatusCode;  //for our use
                if (response.StatusCode == HttpStatusCode.OK || response.StatusCode == HttpStatusCode.Created)
                {
                    strAPICall += strNewLine;
                    strAPICall += "if(response.StatusCode == HttpStatusCode.OK || response.StatusCode == HttpStatusCode.Created) " + strNewLine;
                    strAPICall += "{ " + strNewLine;
                    strAPICall += "\r\tStreamReader sr = new StreamReader(response.GetResponseStream()); " + strNewLine;
                    strAPICall += "\r\tstring strResponse = sr.ReadToEnd(); " + strNewLine;
                    strAPICall += "} " + strNewLine;
                    txtAPICallList.Text = strAPICall;

                    //StreamReader sr = new StreamReader(response.GetResponseStream());
                    //if (sr != null && sr.ReadToEnd().ToString().Trim().Length > 0)
                    //{
                    xmlDoc = new XmlDocument();
                    xmlDoc.Load(response.GetResponseStream());
                    sb = new StringBuilder();
                    settings = new XmlWriterSettings();
                    settings.Indent = true;
                    settings.IndentChars = "  ";
                    settings.NewLineChars = "\r\n";
                    settings.NewLineHandling = NewLineHandling.Replace;
                    writer = XmlWriter.Create(sb, settings);
                    xmlDoc.WriteTo(writer);
                    writer.Close();
                    txtOutputBody.Text = sb.ToString();
                    //}
                    return true;
                }
                else
                    return false;

            }
            catch (Exception ex)
            {
                txtOutputBody.Text = ex.Message;
                throw GetExceptionMsg("AddFile", ex);
            }
            finally
            {
                try { fileStream.Close(); }
                catch (Exception) { }

                try { fileStream.Dispose(); }
                catch (Exception) { }

                try { response.Close(); }
                catch (Exception) { }

                //try { response.Dispose(); }
                //catch (Exception) { }

                response = null;
                httpWebRequest = null;
                xmlDoc = null;
                sb = null;
                settings = null;
                writer = null;
            }
        }

        public bool DownloadFile(string strAPIURL, string strAccessKey, string strFileName)
        {
            WebClient objWC = new WebClient();
            string strAPICall = string.Empty;
            string strNewLine = "\r\n";
            try
            {
                strAPICall += "WebClient objWC = new WebClient(); " + strNewLine;
                strAPICall += strNewLine;

                if (txtProxyServer.Text.Trim().Length <= 0)
                    objWC.Credentials = CredentialCache.DefaultCredentials;
                else
                    objWC.Proxy = GetProxySettings();
                strAPICall += "objWC.Credentials = CredentialCache.DefaultCredentials; " + strNewLine;
                objWC.Headers.Add(HttpRequestHeader.Accept, strXMLContentType);
                strAPICall += "objWC.Headers.Add(HttpRequestHeader.Accept, \"" + strXMLContentType + "\"); " + strNewLine;
                objWC.Headers.Add(HttpRequestHeader.Authorization, strBearer + strAccessKey);
                strAPICall += "objWC.Headers.Add(HttpRequestHeader.Authorization, \"" + strBearer + strAccessKey + "\"); " + strNewLine;
                objWC.Headers.Add("User-Agent", "Firefox");
                strAPICall += "objWC.Headers.Add(\"User-Agent\", \"Firefox\"); " + strNewLine;
                objWC.DownloadFile(strAPIURL, strFileName);
                strAPICall += "objWC.DownloadFile(@\"" + strAPIURL + "\",@\"" + strFileName + "\"); " + strNewLine;
                txtAPICallList.Text = strAPICall;

                if (File.Exists(strFileName))
                    return true;
                else
                    return false;
            }
            catch (Exception ex)
            {
                throw GetExceptionMsg("DownloadFile", ex);
            }
            finally
            {
                if (objWC != null)
                {
                    objWC.Dispose();
                    objWC = null;
                }
            }
        }
        #endregion

        #region API Related Methods
        public static Exception GetExceptionMsg(string strMethodName, Exception objEx)
        {
            try
            {
                return new Exception(Message_0004.Replace("%%%methodname%%%", strMethodName).Replace("%%%line#%%%", GetLineNumber(objEx)) + objEx.Message);
            }
            catch
            {
                //WriteDebugLog("GetExceptionMsg - Error ocured, Exception: " + ex.Message);
                //Do not throw
                return new Exception(string.Empty);
            }
        }

        private static string GetLineNumber(Exception objEx)
        {
            int linenum = 0;
            string strLineNo = string.Empty;
            try
            {
                linenum = Convert.ToInt32(objEx.StackTrace.Substring(objEx.StackTrace.LastIndexOf(":line") + 5));
                strLineNo = linenum > 0 ? "-(Line#:" + linenum.ToString() + ")" : strLineNo;
            }
            catch
            {
                //Stack trace is not available!
            }
            return strLineNo;
        }

        //public string EncodeString(string strValue)
        //{
        //    try
        //    {
        //        return WebUtility.HtmlEncode(strValue);
        //    }
        //    catch (Exception ex)
        //    {
        //        throw GetExceptionMsg("EncodeString", ex);
        //    }
        //}

        //public string DecodeString(string strValue)
        //{
        //    try
        //    {
        //        return WebUtility.HtmlDecode(strValue);
        //    }
        //    catch (Exception ex)
        //    {
        //        throw GetExceptionMsg("DecodeString", ex);
        //    }
        //}

        public string GetFileName(string strPath, FileName objFileName)
        {
            try
            {
                switch (objFileName)
                {
                    case FileName.OnlyExt:
                        return Path.GetExtension(strPath);

                    case FileName.WithoutExt:
                        return Path.GetFileNameWithoutExtension(strPath);

                    case FileName.WithExt:
                        return Path.GetFileName(strPath);
                }
            }
            catch
            {
                try
                {
                    string strFileNameWithExt = string.Empty, strFileExt = string.Empty, strFileNameWithoutExt = string.Empty;

                    if (strFileNameWithExt.Length < 1)
                    {
                        strFileNameWithExt = strPath;
                        if (strFileNameWithExt.LastIndexOf("\\") > -1)
                            strFileNameWithExt = strFileNameWithExt.Substring(strFileNameWithExt.LastIndexOf("\\") + 1);  //File Name with Ext.
                    }

                    if (strFileNameWithExt.LastIndexOf(".") > -1)
                    {
                        strFileExt = strFileNameWithExt.Substring(strFileNameWithExt.LastIndexOf(".")); //File Ext. with "."
                        strFileNameWithoutExt = strFileNameWithExt.Substring(0, strFileNameWithExt.LastIndexOf(".")); //File Name without Ext.
                    }

                    switch (objFileName)
                    {
                        case FileName.OnlyExt:
                            return strFileExt;

                        case FileName.WithoutExt:
                            return strFileNameWithoutExt;

                        case FileName.WithExt:
                            return strFileNameWithExt;
                    }
                }
                catch (Exception ex)
                {
                    throw GetExceptionMsg("GetFileName", ex);
                }
            }

            return string.Empty;
        }

        private void DeleteFile(string strPath)
        {
            try
            {
                File.Delete(strPath);
            }
            catch (Exception ex)
            {
                throw GetExceptionMsg("DeleteFile", ex);
            }
        }
        #endregion
    }

    public class APIInfo
    {
        public int APIID { get; set; }
        public string APIUrl { get; set; }
        public string APIBody { get; set; }
        public string APIDisplayName { get; set; }
        public string RequestType { get; set; }
    }

    public class APIParameter
    {
        public int APIParameterID { get; set; }
        public string ParameterName { get; set; }
        public string ParameterDisplayName { get; set; }
        public string ParameterValue { get; set; }
        public string HelpText { get; set; }
    }
}